"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
const childProcess = require("child_process");
const fs = require("fs");
var spawn = childProcess.spawn;
require("should");
var restartRabbitPath = __dirname + "/../../restart_rabbit.sh";
function restartRabbit(done) {
    var restarter = spawn(restartRabbitPath);
    restarter.stdout.pipe(process.stdout);
    restarter.stderr.pipe(process.stdout);
    restarter.on("close", function () {
        setTimeout(() => {
            done();
        }, 2000);
    });
}
index_1.amqpManager.setConnectionURI("amqp://localhost?heartbeat=2");
index_1.amqpManager.channelManager.randomReconnectionInterval = false;
if (fs.existsSync(restartRabbitPath)) {
    describe("Connection recovery", function () {
        describe("recover events", () => {
            var events;
            var messages = [];
            function listener(message) {
                messages.push(message);
            }
            before(function (done) {
                this.timeout(40e3);
                index_1.amqpManager.channelManager.randomReconnectionInterval = false;
                index_1.amqpManager.reconnect(function () {
                    events = new index_1.amqpManager.events("some-app");
                    events.on("event:recover", listener);
                    events.emit("event:recover", "test-event");
                    setTimeout(() => {
                        messages.length.should.equal(1);
                        messages[0].should.equal("test-event");
                        messages = [];
                        restartRabbit(done);
                    }, 500);
                });
            });
            it("events should go after rebbit restart", (done) => {
                events.emit("event:recover", "after-restart");
                setTimeout(() => {
                    messages.length.should.equal(1);
                    messages[0].should.equal("after-restart");
                    done();
                }, 500);
            });
        });
        describe("recover tasks", () => {
            var tasks;
            var messages = [];
            function processor(message, done) {
                messages.push(message);
                done();
            }
            before(function (done) {
                this.timeout(20e3);
                index_1.amqpManager.reconnect(function () {
                    tasks = index_1.amqpManager.tasks;
                    console.log('Attaching task listener...');
                    tasks.processTask("task:recovered_task", processor, () => {
                        console.log('Attached task listener!');
                        var task = tasks.createTask("task:recovered_task", {
                            title: "test-task"
                        });
                        task.start(() => {
                            setTimeout(() => {
                                messages.length.should.equal(1);
                                messages[0].title.should.equal("test-task");
                                messages = [];
                                restartRabbit(done);
                            }, 1000);
                        });
                    });
                });
            });
            it("tasks should go after rebbit restart", (done) => {
                var task = tasks.createTask("task:recovered_task", {
                    title: "after-restart"
                });
                task.start();
                setTimeout(() => {
                    messages.length.should.equal(1);
                    messages[0].title.should.equal("after-restart");
                    done();
                }, 500);
            });
        });
    });
}
//# sourceMappingURL=connection_recovery.js.map